function PrimeUI:TimerGroup2(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"], data["Height"], data["NickName"] or "Timer Group")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"]) + (data.OffsetX or 0), self.currentHeight)
    end
    valuebar:SetSize(data["Width"], data["Height"])
    valuebar:SetMouseVisible(false)

    local timerGroup = TimerGroupWindow2(self, data, valuebar)

	local triggers = {
		-- Events
		["EventTrigger"] = {},

		-- Effect self
		["Effects"] = {},
		["EffectAdded"] = {},
		["EffectRemoved"] = {},

		-- Effect Group
		["EffectAdded_Group"] = {},

		-- Effect Target
		["EffectAdded_Target"] = {},

		-- Skill
		["Skills"] = {},
		["SkillReady"] = {},

		-- Environment Chat
		["PartyChat"] = {},
		["SayChat"] = {},
		["StandardChat"] = {},

		-- Combat Chat 
		["PlayerHits"] = {},
		["PlayerHeal"] = {},
		["PlayerBenefit"] = {},
		["EnemyHits"] = {},
		["CCBreak"] = {},

		-- Enivorment events
		["DefeatEvent"] = {},

		-- Instance Fights
		["FightStart"] = {},
		["FightPhase"] = {},
		["FightEnd"] = {},
	}

	--timerGroup:Trigger(event, args)
    local timerCode = 0
	for key, value in pairs(data.Timers) do
		for _, trigger in pairs(value.Triggers) do
			if triggers[trigger.Trigger_Type] then
				if not triggers[trigger.Trigger_Type][trigger.Trigger_Key] then triggers[trigger.Trigger_Type][trigger.Trigger_Key] = {} end
				table.insert(triggers[trigger.Trigger_Type][trigger.Trigger_Key], {["Timer_Code"] = timerCode, ["Trigger"] = trigger, ["Timer"] = value})
			end
		end

        if value.Always_Visible then
            timerGroup:Create_Static_Timer(timerCode, value)
        end
        
        timerCode = timerCode + 1
	end

	timerGroup.PrimeUITriggerFunction = function(triggerInfo, eventArgs)
		if not triggerInfo or tablelength(triggerInfo) == 0 then return end
        if not eventArgs or tablelength(eventArgs) == 0 then return end

		for key, value in pairs(triggerInfo.Trigger) do
            eventArgs[key] = value
		end

        if eventArgs.Special_Trigger and eventArgs.Trigger_Time then
            eventArgs.Timer_Duration = eventArgs.Trigger_Time
        end

        timerGroup:Trigger(triggerInfo.Timer_Code, eventArgs, triggerInfo.Timer)
	end

	if tablelength(triggers["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
			local value = triggers["EventTrigger"][event]
			if value then
                local eventArgs = self:BuildArgs("EventTrigger", event, {})
                for k, v in pairs(value) do
                    if not v.Trigger.Use_Target_Filter_List or v.Trigger.Use_Target_Filter_List and v.Trigger.Target_Filter_List[args.Target] then
				        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
			end
        end))
    end

	-- Effect Self
	if tablelength(triggers["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
			local value = triggers["EffectAdded"][name]
            if value and tablelength(value) > 0 then
                local args = self:BuildArgs("EffectAdded", name, effect)
                for k, v in pairs(value) do
                    if v.Trigger.Has_To_Match_Icon and v.Trigger.Match_Icon == tostring(effect:GetIcon()) or not v.Trigger.Has_To_Match_Icon then
				        timerGroup.PrimeUITriggerFunction(v, args)
                    end
                end
            end
        end))
    end

	if tablelength(triggers["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local value = triggers["EffectRemoved"][effect:GetName()]
			if value then
                local args = self:BuildArgs("EffectRemoved", effect:GetName(), effect)
                for k, v in pairs(value) do
                    if v.Trigger.Has_To_Match_Icon and v.Trigger.Match_Icon == tostring(effect:GetIcon()) or not v.Trigger.Has_To_Match_Icon then
				        timerGroup.PrimeUITriggerFunction(v, args)
                    end
                end
            end
        end))
    end

	-- Effect Group
    if tablelength(triggers["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local effectArgs = args.effect
            local value = triggers["EffectAdded_Group"][effectArgs.Name]
            if value then
                local args = self:BuildArgs("EffectAdded_Group", effectArgs.Name, args)
                for k, v in pairs(value) do
                    if v.Trigger.Has_To_Match_Icon and v.Trigger.Match_Icon == tostring(effectArgs.Icon) or not v.Trigger.Has_To_Match_Icon then
                        timerGroup.PrimeUITriggerFunction(v, args)
                    end
                end
            end
        end))
    end

	-- Effect Target
	if tablelength(triggers["EffectAdded_Target"]) > 0 then
		timerGroup.skillsTargeted = true
		table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
		function(args)
			local value = triggers["EffectAdded_Target"][args.Name]
			if value then
                local args = self:BuildArgs("EffectAdded_Target", args.Name, args)
                for k, v in pairs(value) do
                    if not v.Trigger.Use_Target_Filter_List or v.Trigger.Use_Target_Filter_List and v.Trigger.Target_Filter_List[args.Target] then
                        if v.Trigger.Has_To_Match_Icon and v.Trigger.Match_Icon == tostring(args.Icon) or not v.Trigger.Has_To_Match_Icon then
                            timerGroup.PrimeUITriggerFunction(v, args)
                        end
                    end
                end
			end
		end))
	end

	-- Skills
	if tablelength(triggers["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(triggers["Skills"]) do
            if SkillEventHandler:SkillExists(key) then

                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill)  
                    local args = self:BuildArgs("Skills", key, skill)
                    for k, v in pairs(value) do
					    timerGroup.PrimeUITriggerFunction(v, args)
                    end
				end))
            end
        end
    end

	-- Environment Chat
	if tablelength(triggers["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = triggers["StandardChat"][message]
            if value then
                if value then
                    local args = self:BuildArgs("StandardChat", message, {})
                    for k, v in pairs(value) do
					    timerGroup.PrimeUITriggerFunction(v, args)
                    end
                end
            end
        end))
    end

    if tablelength(triggers["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "PartyInfo", function(args)
            local value = triggers["PartyChat"][args.Message]
            if value then
                local eventArgs = self:BuildArgs("PartyChat", args.Message, args)
                for k, v in pairs(value) do
                    timerGroup.PrimeUITriggerFunction(v, eventArgs)
                end
            end
        end))
    end

    if tablelength(triggers["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayInfo", function(args)
            local value = triggers["SayChat"][args.Message]
            if value then
                local eventArgs = self:BuildArgs("SayChat", args.Message, args)
                for k, v in pairs(value) do
                    timerGroup.PrimeUITriggerFunction(v, eventArgs)
                end
            end
        end))
    end

	-- Combat Chat
	if tablelength(triggers["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = triggers["EnemyHits"][args.SkillName]
            if value then
                local eventArgs = self:BuildArgs("EnemyHits", args.SkillName, args)
                for k, v in pairs(value) do
                    if not v.Trigger.Use_Target_Filter_List or v.Trigger.Use_Target_Filter_List and v.Trigger.Target_Filter_List[args.Target] then
				        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))

    end

    if tablelength(triggers["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = triggers["PlayerHits"][args.SkillName]
            if value then
                local eventArgs = self:BuildArgs("PlayerHits", args.SkillName, args)
                for k, v in pairs(value) do
                    if not v.Trigger.Use_Target_Filter_List or v.Trigger.Use_Target_Filter_List and v.Trigger.Target_Filter_List[args.Target] then
				        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))
    end

    if tablelength(triggers["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = triggers["PlayerHeal"][args.SkillName]
            if value then
                local eventArgs = self:BuildArgs("PlayerHeal", args.SkillName, args)
                for k, v in pairs(value) do
                    if not v.Trigger.Use_Target_Filter_List or v.Trigger.Use_Target_Filter_List and v.Trigger.Target_Filter_List[args.Target] then
				        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))
    end

    if tablelength(triggers["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = triggers["PlayerBenefit"][args.SkillName]
            if value then
                local eventArgs = self:BuildArgs("PlayerBenefit", args.SkillName, args)
                for k, v in pairs(value) do
                    if not v.Trigger.Use_Target_Filter_List or v.Trigger.Use_Target_Filter_List and v.Trigger.Target_Filter_List[args.Target] then
				        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))
    end

	-- Enivorment events
	if tablelength(triggers["DefeatEvent"]) > 0 then
        table.insert(self.defeatEventCallback, AddCallback(ChatEventHandler, "Defeated", function(args)
            local value = triggers["DefeatEvent"][args.Name]
            if value then
                local eventArgs = self:BuildArgs("DefeatEvent", args.Name, args)
                for k, v in pairs(value) do
				    timerGroup.PrimeUITriggerFunction(v, eventArgs)
                end
            end
        end))
    end

	-- Instance Fights
	if tablelength(triggers["FightStart"]) > 0 then
        table.insert(self.fightEventStartCallbacks, AddCallback(InstanceController, "FightEventStart", function(args)
            for key, value in pairs(triggers["FightStart"]) do
                local eventArgs = self:BuildArgs("FightStart", args.Boss_Name, args)
                for k, v in pairs(value) do
                    if v.Trigger.Boss_Name == args.Boss_Name and (v.Trigger.Tier_Info == true or  tostring(v.Trigger.Tier_Info) == args.Boss_Tier) then
                        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))
    end

    if tablelength(triggers["FightPhase"]) > 0 then
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            for key, value in pairs(triggers["FightPhase"]) do
                local eventArgs = self:BuildArgs("FightPhase", args.Boss_Name, args)
                for k, v in pairs(value) do
                    if v.Trigger.Boss_Name == args.Boss_Name and (v.Trigger.Tier_Info == true or tostring(v.Trigger.Tier_Info) == args.Boss_Tier) and v.Trigger.Phase == args.Phase then
                        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))
    end

    if tablelength(triggers["FightEnd"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            for key, value in pairs(triggers["FightEnd"]) do
                for k, v in pairs(value) do
                    if v.Trigger.Boss_Name == args.Boss_Name and (v.Trigger.Tier_Info == true or  tostring(v.Trigger.Tier_Info) == args.Boss_Tier) then
                        timerGroup.PrimeUITriggerFunction(v, eventArgs)
                    end
                end
            end
        end))
    end

	if tablelength(data["CCBreak"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "CCBreak", function(args)
        end))
    end

	table.insert(self.tools, timerGroup)

    self.currentHeight = self.currentHeight + data["Height"]
    self.heightSize = self.heightSize + data["Height"]
    table.insert(self.UI, valuebar)
end